@@ -19,13 +19,14 @@
 //! This module contains logic for parsing a python argument list.
 //! See also the macros `py_argparse!`, `py_fn!` and `py_method!`.
 
-use conversion::{RefFromPyObject, ToPyObject};
-use err::{self, PyResult};
-use ffi;
-use objects::{exc, PyDict, PyObject, PyString, PyTuple};
-use python::{Python, PythonObject};
 use std::ptr;
 
+use crate::conversion::{RefFromPyObject, ToPyObject};
+use crate::err::{self, PyResult};
+use crate::ffi;
+use crate::objects::{exc, PyDict, PyObject, PyString, PyTuple};
+use crate::python::{Python, PythonObject};
+
 /// Description of a python parameter; used for `parse_args()`.
 pub struct ParamDescription<'a> {
     /// The name of the parameter.
@@ -37,11 +38,7 @@ pub struct ParamDescription<'a> {
 impl<'a> ParamDescription<'a> {
     /// Name, with leading `r#` stripped.
     pub fn name(&self) -> &str {
-        if self.name.starts_with("r#") {
-            &self.name[2..]
-        } else {
-            self.name
-        }
+        crate::strip_raw!(self.name)
     }
 }
 
@@ -151,7 +148,7 @@ pub fn parse_args(
 ///   If no type is specified, the parameter implicitly uses
 ///   `&PyObject` (format 1), `&PyTuple` (format 4) or `&PyDict` (format 6).
 ///   If a default value is specified, it must be a compile-time constant
-//    of type `ty`.
+///   of type `ty`.
 ///  * `body`: expression of type `PyResult<_>`.
 ///     The extracted argument values are available in this scope.
 ///
@@ -160,20 +157,21 @@ pub fn parse_args(
 /// the body expression and returns of that evaluation.
 /// If extraction fails, `py_argparse!()` returns a failed `PyResult` without evaluating `body`.
 ///
-/// The `py_argparse!()` macro special-cases reference types (when `ty` starts with a `&` token):
-/// In this case, the macro uses the `RefFromPyObject` trait instead of the `FromPyObject` trait.
+/// The `py_argparse!()` macro special-cases reference types (when `ty` starts with a `&` token)
+/// and optional reference types (when `ty` is of the form `Option<&...>`).
+/// In these cases, the macro uses the `RefFromPyObject` trait instead of the `FromPyObject` trait.
 /// When using at least one reference parameter, the `body` block is placed within a closure,
 /// so `return` statements might behave unexpectedly in this case. (this only affects direct use
 /// of `py_argparse!`; `py_fn!` is unaffected as the body there is always in a separate function
 /// from the generated argument-parsing code).
-#[macro_export(local_inner_macros)]
+#[macro_export]
 macro_rules! py_argparse {
     ($py:expr, $fname:expr, $args:expr, $kwargs:expr, $plist:tt $body:block) => {
-        py_argparse_parse_plist! { py_argparse_impl { $py, $fname, $args, $kwargs, $body, } $plist }
+        $crate::py_argparse_parse_plist! { py_argparse_impl { $py, $fname, $args, $kwargs, $body, } $plist }
     };
 }
 
-#[macro_export(local_inner_macros)]
+#[macro_export]
 #[doc(hidden)]
 macro_rules! py_argparse_parse_plist {
     // Parses a parameter-list into a format more suitable for consumption by Rust macros.
@@ -183,30 +181,30 @@ macro_rules! py_argparse_parse_plist {
 
     // Special-case entry-point for empty parameter list:
     { $callback:ident { $($initial_arg:tt)* } ( ) } => {
-        $callback! { $($initial_arg)* [] }
+        $crate::$callback! { $($initial_arg)* [] }
     };
     // Regular entry point for non-empty parameter list:
     { $callback:ident $initial_args:tt ( $( $p:tt )+ ) } => {
         // add trailing comma to plist so that the parsing step can assume every
         // parameter ends with a comma.
-        py_argparse_parse_plist_impl! { $callback $initial_args [] ( $($p)*, ) }
+        $crate::py_argparse_parse_plist_impl! { $callback $initial_args [] ( $($p)*, ) }
     };
 }
 
-#[macro_export(local_inner_macros)]
+#[macro_export]
 #[doc(hidden)]
 macro_rules! py_argparse_parse_plist_impl {
     // TT muncher macro that does the main work for py_argparse_parse_plist!.
 
     // Base case: all parameters handled
     { $callback:ident { $($initial_arg:tt)* } $output:tt ( ) } => {
-        $callback! { $($initial_arg)* $output }
+        $crate::$callback! { $($initial_arg)* $output }
     };
     // Kwargs parameter with reference extraction
     { $callback:ident $initial_args:tt [ $($output:tt)* ]
         ( ** $name:ident : &$t:ty , $($tail:tt)* )
     } => {
-        py_argparse_parse_plist_impl! {
+        $crate::py_argparse_parse_plist_impl! {
             $callback $initial_args
             [ $($output)* { $name:&$t = [ {**} {} {$t} ] } ]
             ($($tail)*)
@@ -216,7 +214,7 @@ macro_rules! py_argparse_parse_plist_impl {
     { $callback:ident $initial_args:tt [ $($output:tt)* ]
         ( ** $name:ident : $t:ty , $($tail:tt)* )
     } => {
-        py_argparse_parse_plist_impl! {
+        $crate::py_argparse_parse_plist_impl! {
             $callback $initial_args
             [ $($output)* { $name:$t = [ {**} {} {} ] } ]
             ($($tail)*)
@@ -226,7 +224,7 @@ macro_rules! py_argparse_parse_plist_impl {
     { $callback:ident $initial_args:tt [ $($output:tt)* ]
         ( ** $name:ident , $($tail:tt)* )
     } => {
-        py_argparse_parse_plist_impl! {
+        $crate::py_argparse_parse_plist_impl! {
             $callback $initial_args
             [ $($output)* { $name:Option<&$crate::PyDict> = [ {**} {} {} ] } ]
             ($($tail)*)
@@ -236,7 +234,7 @@ macro_rules! py_argparse_parse_plist_impl {
     { $callback:ident $initial_args:tt [ $($output:tt)* ]
         ( * $name:ident : &$t:ty , $($tail:tt)* )
     } => {
-        py_argparse_parse_plist_impl! {
+        $crate::py_argparse_parse_plist_impl! {
             $callback $initial_args
             [ $($output)* { $name:&$t = [ {*} {} {$t} ] } ]
             ($($tail)*)
@@ -246,7 +244,7 @@ macro_rules! py_argparse_parse_plist_impl {
     { $callback:ident $initial_args:tt [ $($output:tt)* ]
         ( * $name:ident : $t:ty , $($tail:tt)* )
     } => {
-        py_argparse_parse_plist_impl! {
+        $crate::py_argparse_parse_plist_impl! {
             $callback $initial_args
             [ $($output)* { $name:$t = [ {*} {} {} ] } ]
             ($($tail)*)
@@ -256,7 +254,7 @@ macro_rules! py_argparse_parse_plist_impl {
     { $callback:ident $initial_args:tt [ $($output:tt)* ]
         ( * $name:ident , $($tail:tt)* )
     } => {
-        py_argparse_parse_plist_impl! {
+        $crate::py_argparse_parse_plist_impl! {
             $callback $initial_args
             [ $($output)* { $name:&$crate::PyTuple = [ {*} {} {} ] } ]
             ($($tail)*)
@@ -266,17 +264,27 @@ macro_rules! py_argparse_parse_plist_impl {
     { $callback:ident $initial_args:tt [ $($output:tt)* ]
         ( $name:ident : &$t:ty , $($tail:tt)* )
     } => {
-        py_argparse_parse_plist_impl! {
+        $crate::py_argparse_parse_plist_impl! {
             $callback $initial_args
             [ $($output)* { $name:&$t = [ {} {} {$t} ] } ]
             ($($tail)*)
         }
     };
+    // Maybe None simple parameter with reference extraction
+    { $callback:ident $initial_args:tt [ $($output:tt)* ]
+        ( $name:ident : Option<&$t:ty> , $($tail:tt)* )
+    } => {
+        $crate::py_argparse_parse_plist_impl! {
+            $callback $initial_args
+            [ $($output)* { $name: std::option::Option<&$t> = [ {opt} {} {$t} ] } ]
+            ($($tail)*)
+        }
+    };
     // Simple parameter
     { $callback:ident $initial_args:tt [ $($output:tt)* ]
         ( $name:ident : $t:ty , $($tail:tt)* )
     } => {
-        py_argparse_parse_plist_impl! {
+        $crate::py_argparse_parse_plist_impl! {
             $callback $initial_args
             [ $($output)* { $name:$t = [ {} {} {} ] } ]
             ($($tail)*)
@@ -286,17 +294,27 @@ macro_rules! py_argparse_parse_plist_impl {
     { $callback:ident $initial_args:tt [ $($output:tt)* ]
         ( $name:ident , $($tail:tt)* )
     } => {
-        py_argparse_parse_plist_impl! {
+        $crate::py_argparse_parse_plist_impl! {
             $callback $initial_args
             [ $($output)* { $name:&$crate::PyObject = [ {} {} {} ] } ]
             ($($tail)*)
         }
     };
+    // Maybe None optional parameter with reference extraction
+    { $callback:ident $initial_args:tt [ $($output:tt)* ]
+        ( $name:ident : Option<&$t:ty> = $default:expr , $($tail:tt)* )
+    } => {
+        $crate::py_argparse_parse_plist_impl! {
+            $callback $initial_args
+            [ $($output)* { $name: std::option::Option<&$t> = [ {opt} {$default} {$t} ] } ]
+            ($($tail)*)
+        }
+    };
     // Optional parameter with reference extraction
     { $callback:ident $initial_args:tt [ $($output:tt)* ]
         ( $name:ident : &$t:ty = $default:expr, $($tail:tt)* )
     } => {
-        py_argparse_parse_plist_impl! {
+        $crate::py_argparse_parse_plist_impl! {
             $callback $initial_args
             [ $($output)* { $name:&$t = [ {} {$default} {$t} ] } ]
             ($($tail)*)
@@ -306,7 +324,7 @@ macro_rules! py_argparse_parse_plist_impl {
     { $callback:ident $initial_args:tt [ $($output:tt)* ]
         ( $name:ident : $t:ty = $default:expr , $($tail:tt)* )
     } => {
-        py_argparse_parse_plist_impl! {
+        $crate::py_argparse_parse_plist_impl! {
             $callback $initial_args
             [ $($output)* { $name:$t = [ {} {$default} {} ] } ]
             ($($tail)*)
@@ -316,7 +334,7 @@ macro_rules! py_argparse_parse_plist_impl {
 
 // The main py_argparse!() macro, except that it expects the parameter-list
 // in the output format of py_argparse_parse_plist!().
-#[macro_export(local_inner_macros)]
+#[macro_export]
 #[doc(hidden)]
 macro_rules! py_argparse_impl {
     // special case: function signature is (*args, **kwargs),
@@ -340,11 +358,11 @@ macro_rules! py_argparse_impl {
     ) => {{
         const PARAMS: &'static [$crate::argparse::ParamDescription<'static>] = &[
             $(
-                py_argparse_param_description! { $pname : $ptype = $detail }
+                $crate::py_argparse_param_description! { $pname : $ptype = $detail }
             ),*
         ];
         let py: $crate::Python = $py;
-        let mut output = [$( py_replace_expr!($pname None) ),*];
+        let mut output = [$( $crate::py_replace_expr!($pname None) ),*];
         match $crate::argparse::parse_args(py, $fname, PARAMS, $args, $kwargs, &mut output) {
             Ok(()) => {
                 // Experimental slice pattern syntax would be really nice here (#23121)
@@ -354,7 +372,7 @@ macro_rules! py_argparse_impl {
                 // We'll have to generate a bunch of nested `match` statements
                 // (at least until we can use ? + catch, assuming that will be hygienic wrt. macros),
                 // so use a recursive helper macro for that:
-                py_argparse_extract!( py, _iter, $body,
+                $crate::py_argparse_extract!( py, _iter, $body,
                     [ $( { $pname : $ptype = $detail } )* ])
             },
             Err(e) => Err(e)
@@ -363,14 +381,14 @@ macro_rules! py_argparse_impl {
 }
 
 // Like py_argparse_impl!(), but accepts `*mut ffi::PyObject` for $args and $kwargs.
-#[macro_export(local_inner_macros)]
+#[macro_export]
 #[doc(hidden)]
 macro_rules! py_argparse_raw {
     ($py:ident, $fname:expr, $args:expr, $kwargs:expr, $plist:tt $body:block) => {{
         let args: $crate::PyTuple =
             $crate::PyObject::from_borrowed_ptr($py, $args).unchecked_cast_into();
         let kwargs: Option<$crate::PyDict> = $crate::argparse::get_kwargs($py, $kwargs);
-        let ret = py_argparse_impl!($py, $fname, &args, kwargs.as_ref(), $body, $plist);
+        let ret = $crate::py_argparse_impl!($py, $fname, &args, kwargs.as_ref(), $body, $plist);
         $crate::PyDrop::release_ref(args, $py);
         $crate::PyDrop::release_ref(kwargs, $py);
         ret
@@ -391,14 +409,14 @@ pub unsafe fn get_kwargs(py: Python, ptr: *mut ffi::PyObject) -> Option<PyDict>
 #[doc(hidden)]
 macro_rules! py_argparse_param_description {
     // normal parameter
-    { $pname:ident : $ptype:ty = [ {} {} $rtype:tt ] } => (
+    { $pname:ident : $ptype:ty = [ $info:tt {} $rtype:tt ] } => (
         $crate::argparse::ParamDescription {
             name: stringify!($pname),
             is_optional: false
         }
     );
     // optional parameters
-    { $pname:ident : $ptype:ty = [ {} {$default:expr} {$($rtype:tt)*} ] } => (
+    { $pname:ident : $ptype:ty = [ $info:tt {$default:expr} $rtype:tt ] } => (
         $crate::argparse::ParamDescription {
             name: stringify!($pname),
             is_optional: true
@@ -406,7 +424,7 @@ macro_rules! py_argparse_param_description {
     );
 }
 
-#[macro_export(local_inner_macros)]
+#[macro_export]
 #[doc(hidden)]
 macro_rules! py_argparse_extract {
     // base case
@@ -418,7 +436,7 @@ macro_rules! py_argparse_extract {
         // First unwrap() asserts the iterated sequence is long enough (which should be guaranteed);
         // second unwrap() asserts the parameter was not missing (which fn parse_args already checked for).
         match <$ptype as $crate::FromPyObject>::extract($py, $iter.next().unwrap().as_ref().unwrap()) {
-            Ok($pname) => py_argparse_extract!($py, $iter, $body, [$($tail)*]),
+            Ok($pname) => $crate::py_argparse_extract!($py, $iter, $body, [$($tail)*]),
             Err(e) => Err(e)
         }
     };
@@ -430,18 +448,36 @@ macro_rules! py_argparse_extract {
         // second unwrap() asserts the parameter was not missing (which fn parse_args already checked for).
         match <$rtype as $crate::RefFromPyObject>::with_extracted($py,
             $iter.next().unwrap().as_ref().unwrap(),
-            |$pname: $ptype| py_argparse_extract!($py, $iter, $body, [$($tail)*])
+            |$pname: $ptype| $crate::py_argparse_extract!($py, $iter, $body, [$($tail)*])
         ) {
             Ok(v) => v,
             Err(e) => Err(e)
         }
     };
+    // maybe none parameter with reference extraction
+    ( $py:expr, $iter:expr, $body:block,
+        [ { $pname:ident : $ptype:ty = [ {opt} {} {$rtype:ty} ] } $($tail:tt)* ]
+    ) => {{
+        // First unwrap() asserts the iterated sequence is long enough (which should be guaranteed);
+        // second unwrap() asserts the parameter was not missing (which fn parse_args already checked for).
+        let v = $iter.next().unwrap().as_ref().unwrap();
+        let mut c = |$pname: $ptype| $crate::py_argparse_extract!($py, $iter, $body, [$($tail)*]);
+        let r = if v.as_ptr() == unsafe { $crate::_detail::ffi::Py_None() } {
+            Ok(c(None))
+        } else {
+            <$rtype as $crate::RefFromPyObject>::with_extracted($py, v, |r: &$rtype| c(Some(r)))
+        };
+        match r {
+            Ok(v) => v,
+            Err(e) => Err(e)
+        }
+    }};
     // optional parameter
     ( $py:expr, $iter:expr, $body:block,
         [ { $pname:ident : $ptype:ty = [ {} {$default:expr} {} ] } $($tail:tt)* ]
     ) => {
         match $iter.next().unwrap().as_ref().map(|obj| obj.extract::<_>($py)).unwrap_or(Ok($default)) {
-            Ok($pname) => py_argparse_extract!($py, $iter, $body, [$($tail)*]),
+            Ok($pname) => $crate::py_argparse_extract!($py, $iter, $body, [$($tail)*]),
             Err(e) => Err(e)
         }
     };
@@ -452,9 +488,19 @@ macro_rules! py_argparse_extract {
         //unwrap() asserts the iterated sequence is long enough (which should be guaranteed);
         $crate::argparse::with_extracted_or_default($py,
             $iter.next().unwrap().as_ref(),
-            |$pname: $ptype| py_argparse_extract!($py, $iter, $body, [$($tail)*]),
+            |$pname: $ptype| $crate::py_argparse_extract!($py, $iter, $body, [$($tail)*]),
             $default)
     };
+    // maybe none optional parameter with reference extraction
+    ( $py:expr, $iter:expr, $body:block,
+        [ { $pname:ident : $ptype:ty = [ {opt} {$default:expr} {$rtype:ty} ] } $($tail:tt)* ]
+    ) => {{
+        //unwrap() asserts the iterated sequence is long enough (which should be guaranteed);
+        $crate::argparse::with_extracted_optional_or_default($py,
+            $iter.next().unwrap().as_ref(),
+            |$pname: $ptype| $crate::py_argparse_extract!($py, $iter, $body, [$($tail)*]),
+            $default)
+    }};
 }
 
 #[doc(hidden)] // used in py_argparse_extract!() macro
@@ -477,11 +523,37 @@ where
     }
 }
 
+#[doc(hidden)] // used in py_argparse_extract!() macro
+pub fn with_extracted_optional_or_default<P: ?Sized, R, F>(
+    py: Python,
+    obj: Option<&PyObject>,
+    f: F,
+    default: Option<&'static P>,
+) -> PyResult<R>
+where
+    F: FnOnce(Option<&P>) -> PyResult<R>,
+    P: RefFromPyObject,
+{
+    match obj {
+        Some(obj) => {
+            if obj.as_ptr() == unsafe { crate::ffi::Py_None() } {
+                f(None)
+            } else {
+                match P::with_extracted(py, obj, |p| f(Some(p))) {
+                    Ok(result) => result,
+                    Err(e) => Err(e),
+                }
+            }
+        }
+        None => f(default),
+    }
+}
+
 #[cfg(test)]
 mod test {
-    use conversion::ToPyObject;
-    use objects::PyTuple;
-    use python::{Python, PythonObject};
+    use crate::conversion::ToPyObject;
+    use crate::objects::PyTuple;
+    use crate::python::{Python, PythonObject};
 
     #[test]
     pub fn test_parse() {
