@@ -33,8 +33,7 @@ in all function bodies.
 
 # Example
 ```
-#[macro_use] extern crate cpython;
-use cpython::{Python, PyResult, PyDict};
+use cpython::{Python, PyResult, PyDict, py_class};
 
 py_class!(class MyType |py| {
     data number: i32;
@@ -167,6 +166,30 @@ Declares a static method callable from Python.
 * For details on `parameter-list`, see the documentation of `py_argparse!()`.
 * The return type must be `PyResult<T>` for some `T` that implements `ToPyObject`.
 
+## Properties
+`@property def property_name(&self) -> PyResult<...> { ... }`
+
+`@property_name.setter def set_method_name(&self, value: Option<impl FromPyObject>) -> PyResult<()> { ... }`
+
+Declares a Python data attribute backed by Rust methods to
+get its value and, optionally, to set or delete it.
+
+### Setter details
+
+* The setter is optional.  If omitted, the attribute will be read-only
+  and any setting or deleting attempt will raise `AttributeError`.
+* Unlike Python, the setter method name must be different from the property name.
+  The setter method name is used to call the setter from Rust.
+* A `None` value represents that the property is being deleted, for instance
+  with Python `del`.
+  As with Python properties, what should happen is entirely up to the implementation.
+* The value type can be any type that implements `FromPyObject`, or a reference or
+  optional reference to any type that implements `RefFromPyObject`.  In the latter
+  case, the type of the value is `Option<Option<&impl RefFromPyObject>>`, where
+  `None` means the property is being deleted, `Some(None)` means the property is
+  being set to Python `None`, and `Some(Some(value))` means the property is being
+  set to the given value.
+
 ## __new__
 `def __new__(cls, parameter-list) -> PyResult<...> { ... }`
 
@@ -198,9 +221,8 @@ as every cycle must contain at least one mutable reference.
 Example:
 
 ```
-#[macro_use] extern crate cpython;
 use std::{mem, cell};
-use cpython::{PyObject, PyDrop};
+use cpython::{PyObject, PyDrop, py_class};
 
 py_class!(class ClassWithGCSupport |py| {
     data obj: cell::RefCell<Option<PyObject>>;
@@ -251,9 +273,8 @@ Iterators can be defined using the Python special methods `__iter__` and `__next
 Example:
 
 ```
-#[macro_use] extern crate cpython;
 use std::cell::RefCell;
-use cpython::{PyObject, PyClone, PyResult};
+use cpython::{PyObject, PyClone, PyResult, py_class};
 
 py_class!(class MyIterator |py| {
     data iter: RefCell<Box<Iterator<Item=PyObject> + Send>>;
@@ -425,7 +446,7 @@ py_class!(class MyIterator |py| {
 #[macro_export]
 macro_rules! py_class {
     (class $class:ident |$py: ident| { $( $body:tt )* }) => (
-        py_class_impl! {
+        $crate::py_class_impl! {
             { $( $body )* }
             $class $py
             /* info: */ {
@@ -451,10 +472,11 @@ macro_rules! py_class {
             }
             /* impls: */ { /* impl body */ }
             /* members: */ { /* ident = expr; */ }
+            /* props: */ { [ /* getters */ ] [ /* setters */ ] }
         }
     );
     (pub class $class:ident |$py: ident| { $( $body:tt )* }) => (
-        py_class_impl! {
+        $crate::py_class_impl! {
             { $( $body )* }
             $class $py
             /* info: */ {
@@ -480,6 +502,7 @@ macro_rules! py_class {
             }
             /* impls: */ { /* impl body */ }
             /* members: */ { /* ident = expr; */ }
+            /* props: */ { [ /* getters */ ] [ /* setters */ ] }
         }
     );
 }
@@ -489,7 +512,7 @@ macro_rules! py_class {
 macro_rules! py_class_impl_item {
     { $class:ident, $py:ident, $name:ident( $( $selfarg:tt )* )
         $res_type:ty; $body:block [ $( { $pname:ident : $ptype:ty = $detail:tt } )* ]
-    } => { py_coerce_item! {
+    } => { $crate::py_coerce_item! {
         impl $class {
             pub fn $name($( $selfarg )* $py: $crate::Python $( , $pname: $ptype )* )
             -> $res_type $body
