@@ -16,20 +16,21 @@
 // OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-use conversion::ToPyObject;
-use err::{PyErr, PyResult};
-use exc;
-use ffi;
-use function::CallbackConverter;
 use libc::{c_char, c_int};
-use objects::PyObject;
-use py_class::CompareOp;
-use python::{Python, PythonObject};
 use std::ffi::CString;
 use std::{isize, mem, ptr};
-use Py_hash_t;
 
-#[macro_export(local_inner_macros)]
+use crate::conversion::ToPyObject;
+use crate::err::{PyErr, PyResult};
+use crate::exc;
+use crate::ffi;
+use crate::function::CallbackConverter;
+use crate::objects::PyObject;
+use crate::py_class::CompareOp;
+use crate::python::{Python, PythonObject};
+use crate::Py_hash_t;
+
+#[macro_export]
 #[doc(hidden)]
 macro_rules! py_class_type_object_static_init {
     ($class_name:ident,
@@ -44,8 +45,8 @@ macro_rules! py_class_type_object_static_init {
         $crate::_detail::ffi::PyTypeObject {
             $( $slot_name : $slot_value, )*
             tp_dealloc: Some($crate::py_class::slots::tp_dealloc_callback::<$class_name>),
-            tp_flags: py_class_type_object_flags!($gc),
-            tp_traverse: py_class_tp_traverse!($class_name, $gc),
+            tp_flags: $crate::py_class_type_object_flags!($gc),
+            tp_traverse: $crate::py_class_tp_traverse!($class_name, $gc),
             ..
             $crate::_detail::ffi::PyTypeObject_INIT
         }
@@ -75,7 +76,7 @@ pub const TPFLAGS_DEFAULT: ::libc::c_long = ffi::Py_TPFLAGS_DEFAULT | ffi::Py_TP
 #[cfg(feature = "python3-sys")]
 pub const TPFLAGS_DEFAULT: ::libc::c_ulong = ffi::Py_TPFLAGS_DEFAULT;
 
-#[macro_export(local_inner_macros)]
+#[macro_export]
 #[doc(hidden)]
 macro_rules! py_class_type_object_dynamic_init {
     // initialize those fields of PyTypeObject that we couldn't initialize statically
@@ -87,20 +88,21 @@ macro_rules! py_class_type_object_dynamic_init {
             $as_mapping:tt
             $setdelitem:tt
         }
+        $props:tt
     ) => {
         unsafe {
             $type_object.init_ob_type(&mut $crate::_detail::ffi::PyType_Type);
-            $type_object.tp_name = $crate::py_class::slots::build_tp_name(
-                $module_name,
-                _cpython__py_class__slots__stringify!($class),
-            );
+            $type_object.tp_name =
+                $crate::py_class::slots::build_tp_name($module_name, stringify!($class));
             $type_object.tp_basicsize = <$class as $crate::py_class::BaseObject>::size()
                 as $crate::_detail::ffi::Py_ssize_t;
         }
         // call slot macros outside of unsafe block
-        *(unsafe { &mut $type_object.tp_as_sequence }) = py_class_as_sequence!($as_sequence);
-        *(unsafe { &mut $type_object.tp_as_number }) = py_class_as_number!($as_number);
-        py_class_as_mapping!($type_object, $as_mapping, $setdelitem);
+        *(unsafe { &mut $type_object.tp_as_sequence }) =
+            $crate::py_class_as_sequence!($as_sequence);
+        *(unsafe { &mut $type_object.tp_as_number }) = $crate::py_class_as_number!($as_number);
+        $crate::py_class_as_mapping!($type_object, $as_mapping, $setdelitem);
+        *(unsafe { &mut $type_object.tp_getset }) = $crate::py_class_tp_getset!($class, $props);
     };
 }
 
@@ -117,14 +119,14 @@ pub unsafe extern "C" fn tp_dealloc_callback<T>(obj: *mut ffi::PyObject)
 where
     T: super::BaseObject,
 {
-    let guard = ::function::AbortOnDrop("Cannot unwind out of tp_dealloc");
+    let guard = crate::function::AbortOnDrop("Cannot unwind out of tp_dealloc");
     let py = Python::assume_gil_acquired();
     let r = T::dealloc(py, obj);
     mem::forget(guard);
     r
 }
 
-#[macro_export(local_inner_macros)]
+#[macro_export]
 #[doc(hidden)]
 macro_rules! py_class_wrap_newfunc {
     ($class:ident :: $f:ident [ $( { $pname:ident : $ptype:ty = $detail:tt } )* ]) => {{
@@ -134,11 +136,11 @@ macro_rules! py_class_wrap_newfunc {
             kwargs: *mut $crate::_detail::ffi::PyObject)
         -> *mut $crate::_detail::ffi::PyObject
         {
-            const LOCATION: &'static str = _cpython__py_class__slots__concat!(_cpython__py_class__slots__stringify!($class), ".", _cpython__py_class__slots__stringify!($f), "()");
+            const LOCATION: &'static str = concat!(stringify!($class), ".", stringify!($f), "()");
             $crate::_detail::handle_callback(
                 LOCATION, $crate::_detail::PyObjectCallbackConverter,
                 |py| {
-                    py_argparse_raw!(py, Some(LOCATION), args, kwargs,
+                    $crate::py_argparse_raw!(py, Some(LOCATION), args, kwargs,
                         [ $( { $pname : $ptype = $detail } )* ]
                         {
                             let cls = $crate::PyType::from_type_ptr(py, cls);
@@ -182,7 +184,7 @@ macro_rules! py_class_as_number {
     }}
 }
 
-#[macro_export(local_inner_macros)]
+#[macro_export]
 #[doc(hidden)]
 macro_rules! py_class_as_mapping {
     ( $type_object:ident, [], [
@@ -211,11 +213,11 @@ macro_rules! py_class_as_mapping {
             val: *mut $crate::_detail::ffi::PyObject
         ) -> $crate::_detail::libc::c_int {
             if val.is_null() {
-                py_class_mp_ass_subscript!($delitem, slf,
+                $crate::py_class_mp_ass_subscript!($delitem, slf,
                     b"Subscript assignment not supported by %.200s\0",
                     key)
             } else {
-                py_class_mp_ass_subscript!($setitem, slf,
+                $crate::py_class_mp_ass_subscript!($setitem, slf,
                     b"Subscript deletion not supported by %.200s\0",
                     key, val)
             }
@@ -251,6 +253,62 @@ pub unsafe fn mp_ass_subscript_error(o: *mut ffi::PyObject, err: &[u8]) -> c_int
     -1
 }
 
+#[macro_export]
+#[doc(hidden)]
+macro_rules! py_class_call_slot_impl_with_ref {
+    (
+        $py:ident,
+        $slf:ident,
+        $f:ident,
+        $arg:ident: [ Option<&$arg_type:ty> ],
+        $arg_normal:expr,
+        $arg_if_none:expr,
+        $arg_if_some:expr
+        $(, $extra_arg:ident)*
+    ) => {{
+        if $arg.as_ptr() == unsafe { $crate::_detail::ffi::Py_None() } {
+            Ok($slf.$f($py, $arg_if_none $(, $extra_arg)*))
+        } else {
+            <$arg_type as $crate::RefFromPyObject>::with_extracted(
+                $py,
+                &$arg,
+                |$arg: &$arg_type| $slf.$f($py, $arg_if_some $(, $extra_arg)*)
+            )
+        }
+    }};
+
+    (
+        $py:ident,
+        $slf:ident,
+        $f:ident,
+        $arg:ident: [ &$arg_type:ty ],
+        $arg_normal:expr,
+        $arg_if_none:expr,
+        $arg_if_some:expr
+        $(, $extra_arg:ident)*
+    ) => {{
+        <$arg_type as $crate::RefFromPyObject>::with_extracted(
+            $py,
+            &$arg,
+            |$arg: &$arg_type| $slf.$f($py, $arg_normal $(, $extra_arg)*)
+        )
+    }};
+
+    (
+        $py:ident,
+        $slf:ident,
+        $f:ident,
+        $arg:ident: [ $arg_type:ty ],
+        $arg_normal:expr,
+        $arg_if_none:expr,
+        $arg_if_some:expr
+        $(, $extra_arg:ident)*
+    ) => {{
+        <$arg_type as $crate::FromPyObject>::extract($py, &$arg)
+            .map(|$arg| $slf.$f($py, $arg_normal $(, $extra_arg)*))
+    }};
+}
+
 #[macro_export]
 #[doc(hidden)]
 macro_rules! py_class_unary_slot {
@@ -272,7 +330,7 @@ macro_rules! py_class_unary_slot {
 #[macro_export]
 #[doc(hidden)]
 macro_rules! py_class_binary_slot {
-    ($class:ident :: $f:ident, $arg_type:ty, $res_type:ty, $conv:expr) => {{
+    ($class:ident :: $f:ident, $arg_type:tt, $res_type:ty, $conv:expr) => {{
         unsafe extern "C" fn wrap_binary(
             slf: *mut $crate::_detail::ffi::PyObject,
             arg: *mut $crate::_detail::ffi::PyObject,
@@ -282,8 +340,16 @@ macro_rules! py_class_binary_slot {
                 let slf =
                     $crate::PyObject::from_borrowed_ptr(py, slf).unchecked_cast_into::<$class>();
                 let arg = $crate::PyObject::from_borrowed_ptr(py, arg);
-                let ret = match <$arg_type as $crate::FromPyObject>::extract(py, &arg) {
-                    Ok(arg) => slf.$f(py, arg),
+                let ret = match py_class_call_slot_impl_with_ref!(
+                    py,
+                    slf,
+                    $f,
+                    arg: $arg_type,
+                    arg,
+                    None,
+                    Some(arg)
+                ) {
+                    Ok(r) => r,
                     Err(e) => Err(e),
                 };
                 $crate::PyDrop::release_ref(arg, py);
@@ -298,7 +364,7 @@ macro_rules! py_class_binary_slot {
 #[macro_export]
 #[doc(hidden)]
 macro_rules! py_class_ternary_slot {
-    ($class:ident :: $f:ident, $arg1_type:ty, $arg2_type:ty, $res_type:ty, $conv:expr) => {{
+    ($class:ident :: $f:ident, $arg1_type:tt, $arg2_type:ty, $res_type:ty, $conv:expr) => {{
         unsafe extern "C" fn wrap_binary(
             slf: *mut $crate::_detail::ffi::PyObject,
             arg1: *mut $crate::_detail::ffi::PyObject,
@@ -310,11 +376,22 @@ macro_rules! py_class_ternary_slot {
                     $crate::PyObject::from_borrowed_ptr(py, slf).unchecked_cast_into::<$class>();
                 let arg1 = $crate::PyObject::from_borrowed_ptr(py, arg1);
                 let arg2 = $crate::PyObject::from_borrowed_ptr(py, arg2);
-                let ret = match <$arg1_type as $crate::FromPyObject>::extract(py, &arg1) {
-                    Ok(arg1) => match <$arg2_type as $crate::FromPyObject>::extract(py, &arg2) {
-                        Ok(arg2) => slf.$f(py, arg1, arg2),
-                        Err(e) => Err(e),
-                    },
+                let ret = match <$arg2_type as $crate::FromPyObject>::extract(py, &arg2) {
+                    Ok(arg2) => {
+                        match py_class_call_slot_impl_with_ref!(
+                            py,
+                            slf,
+                            $f,
+                            arg1: $arg1_type,
+                            arg1,
+                            None,
+                            Some(arg1),
+                            arg2
+                        ) {
+                            Ok(r) => r,
+                            Err(e) => Err(e),
+                        }
+                    }
                     Err(e) => Err(e),
                 };
                 $crate::PyDrop::release_ref(arg1, py);
@@ -350,7 +427,7 @@ pub fn extract_op(py: Python, op: c_int) -> PyResult<CompareOp> {
 #[macro_export]
 #[doc(hidden)]
 macro_rules! py_class_richcompare_slot {
-    ($class:ident :: $f:ident, $arg_type:ty, $res_type:ty, $conv:expr) => {{
+    ($class:ident :: $f:ident, $arg_type:tt, $res_type:ty, $conv:expr) => {{
         unsafe extern "C" fn tp_richcompare(
             slf: *mut $crate::_detail::ffi::PyObject,
             arg: *mut $crate::_detail::ffi::PyObject,
@@ -362,12 +439,21 @@ macro_rules! py_class_richcompare_slot {
                     $crate::PyObject::from_borrowed_ptr(py, slf).unchecked_cast_into::<$class>();
                 let arg = $crate::PyObject::from_borrowed_ptr(py, arg);
                 let ret = match $crate::py_class::slots::extract_op(py, op) {
-                    Ok(op) => match <$arg_type as $crate::FromPyObject>::extract(py, &arg) {
-                        Ok(arg) => slf
-                            .$f(py, arg, op)
-                            .map(|res| res.into_py_object(py).into_object()),
-                        Err(_) => Ok(py.NotImplemented()),
-                    },
+                    Ok(op) => {
+                        match py_class_call_slot_impl_with_ref!(
+                            py,
+                            slf,
+                            $f,
+                            arg: $arg_type,
+                            arg,
+                            None,
+                            Some(arg),
+                            op
+                        ) {
+                            Ok(r) => r.map(|r| r.into_py_object(py).into_object()),
+                            Err(e) => Ok(py.NotImplemented()),
+                        }
+                    }
                     Err(_) => Ok(py.NotImplemented()),
                 };
                 $crate::PyDrop::release_ref(arg, py);
@@ -383,7 +469,7 @@ macro_rules! py_class_richcompare_slot {
 #[macro_export]
 #[doc(hidden)]
 macro_rules! py_class_contains_slot {
-    ($class:ident :: $f:ident, $arg_type:ty) => {{
+    ($class:ident :: $f:ident, $arg_type:tt) => {{
         unsafe extern "C" fn sq_contains(
             slf: *mut $crate::_detail::ffi::PyObject,
             arg: *mut $crate::_detail::ffi::PyObject,
@@ -396,8 +482,16 @@ macro_rules! py_class_contains_slot {
                     let slf = $crate::PyObject::from_borrowed_ptr(py, slf)
                         .unchecked_cast_into::<$class>();
                     let arg = $crate::PyObject::from_borrowed_ptr(py, arg);
-                    let ret = match <$arg_type as $crate::FromPyObject>::extract(py, &arg) {
-                        Ok(arg) => slf.$f(py, arg),
+                    let ret = match py_class_call_slot_impl_with_ref!(
+                        py,
+                        slf,
+                        $f,
+                        arg: $arg_type,
+                        arg,
+                        None,
+                        Some(arg)
+                    ) {
+                        Ok(r) => r,
                         Err(e) => $crate::py_class::slots::type_error_to_false(py, e),
                     };
                     $crate::PyDrop::release_ref(arg, py);
@@ -569,7 +663,7 @@ impl CallbackConverter<bool> for BoolConverter {
     }
 }
 
-#[macro_export(local_inner_macros)]
+#[macro_export]
 #[doc(hidden)]
 macro_rules! py_class_call_slot {
     ($class:ident :: $f:ident [ $( { $pname:ident : $ptype:ty = $detail:tt } )* ]) => {{
@@ -579,11 +673,11 @@ macro_rules! py_class_call_slot {
             kwargs: *mut $crate::_detail::ffi::PyObject)
         -> *mut $crate::_detail::ffi::PyObject
         {
-            const LOCATION: &'static str = _cpython__py_class__slots__concat!(_cpython__py_class__slots__stringify!($class), ".", _cpython__py_class__slots__stringify!($f), "()");
+            const LOCATION: &'static str = concat!(stringify!($class), ".", stringify!($f), "()");
             $crate::_detail::handle_callback(
                 LOCATION, $crate::_detail::PyObjectCallbackConverter,
                 |py| {
-                    py_argparse_raw!(py, Some(LOCATION), args, kwargs,
+                    $crate::py_argparse_raw!(py, Some(LOCATION), args, kwargs,
                         [ $( { $pname : $ptype = $detail } )* ]
                         {
                             let slf = $crate::PyObject::from_borrowed_ptr(py, slf).unchecked_cast_into::<$class>();
@@ -611,18 +705,117 @@ pub unsafe extern "C" fn sq_item(
     ret
 }
 
-#[doc(hidden)]
 #[macro_export]
-macro_rules! _cpython__py_class__slots__concat {
-    ($($inner:tt)*) => {
-        concat! { $($inner)* }
-    }
+#[doc(hidden)]
+macro_rules! py_class_prop_getter {
+    ($class:ident :: $f:ident) => {{
+        unsafe extern "C" fn wrap_getter(
+            slf: *mut $crate::_detail::ffi::PyObject,
+            _closure: *mut $crate::_detail::libc::c_void,
+        ) -> *mut $crate::_detail::ffi::PyObject {
+            const LOCATION: &'static str = concat!(stringify!($class), ".", stringify!($f), "{}");
+            $crate::_detail::handle_callback(
+                LOCATION,
+                $crate::_detail::PyObjectCallbackConverter,
+                |py| {
+                    let slf = $crate::PyObject::from_borrowed_ptr(py, slf)
+                        .unchecked_cast_into::<$class>();
+                    let ret = slf.$f(py);
+                    $crate::PyDrop::release_ref(slf, py);
+                    ret
+                },
+            )
+        }
+        Some(wrap_getter)
+    }};
 }
 
-#[doc(hidden)]
 #[macro_export]
-macro_rules! _cpython__py_class__slots__stringify {
-    ($($inner:tt)*) => {
-        stringify! { $($inner)* }
-    }
+#[doc(hidden)]
+macro_rules! py_class_prop_setter {
+    ($class:ident :: $f:ident, $value_type:tt) => {{
+        unsafe extern "C" fn wrap_setter(
+            slf: *mut $crate::_detail::ffi::PyObject,
+            obj: *mut $crate::_detail::ffi::PyObject,
+            _closure: *mut $crate::_detail::libc::c_void,
+        ) -> $crate::_detail::libc::c_int {
+            const LOCATION: &'static str = concat!(stringify!($class), ".", stringify!($f), "{}");
+            $crate::_detail::handle_callback(
+                LOCATION,
+                $crate::py_class::slots::UnitCallbackConverter,
+                |py| {
+                    let slf = $crate::PyObject::from_borrowed_ptr(py, slf)
+                        .unchecked_cast_into::<$class>();
+                    let ret = if obj.is_null() {
+                        slf.$f(py, None)
+                    } else {
+                        let obj = $crate::PyObject::from_borrowed_ptr(py, obj);
+                        let ret = match py_class_call_slot_impl_with_ref!(
+                            py,
+                            slf,
+                            $f,
+                            obj: $value_type,
+                            Some(obj),
+                            Some(None),
+                            Some(Some(obj))
+                        ) {
+                            Ok(r) => r,
+                            Err(e) => Err(e),
+                        };
+                        $crate::PyDrop::release_ref(obj, py);
+                        ret
+                    };
+                    $crate::PyDrop::release_ref(slf, py);
+                    ret
+                },
+            )
+        }
+        Some(wrap_setter)
+    }};
+}
+
+#[macro_export]
+#[doc(hidden)]
+macro_rules! py_class_tp_getset {
+    ( $class:ident, { [] [] } ) => { 0 as *mut $crate::_detail::ffi::PyGetSetDef };
+    (
+        $class:ident,
+        {
+            [ $( { $doc:expr } $getter_name:ident: $prop_type:ty, )* ]
+            [ $( $setter_name:ident: $value_type:tt => $setter_setter:ident, )* ]
+        }
+    ) => {{
+        let mut index = 0usize;
+        $( let $getter_name = index; index += 1; )*
+        unsafe {
+            static mut GETSET: &mut [$crate::_detail::ffi::PyGetSetDef] = &mut [
+                $($crate::_detail::ffi::PyGetSetDef {
+                    name: 0 as *mut _,
+                    get: py_class_prop_getter!($class::$getter_name),
+                    set: None,
+                    doc: 0 as *mut _,
+                    closure: 0 as *mut _,
+                },)*
+                $crate::_detail::ffi::PyGetSetDef {
+                    name: 0 as *mut _,
+                    get: None,
+                    set: None,
+                    doc: 0 as *mut _,
+                    closure: 0 as *mut _,
+                }
+            ];
+            $(
+                GETSET[$getter_name].name = $crate::strip_raw!(
+                    concat!(stringify!($getter_name), "\0")
+                ).as_ptr() as *mut _;
+                if !$doc.is_empty() {
+                    GETSET[$getter_name].doc = concat!($doc, "\0").as_ptr() as *mut _;
+                }
+            )*
+            $(
+                GETSET[$setter_name].set = py_class_prop_setter!($class::$setter_setter, $value_type);
+            )*
+            GETSET.as_ptr() as *mut _
+        }
+    }};
 }
